<?php

namespace Schaffen\Gef\Services;

use App\Http\Controllers\Api\Ged\AnexoServices;
use App\Http\Services\Estoque\LocaisEstoque\EntradaService;
use App\Http\Services\Estoque\LocaisEstoque\TransferenciaService;
use App\Http\Utils\SCHArray;
use Prettus\Validator\Exceptions\ValidatorException;
use Schaffen\Gef\Models\ApontamentoVeiculoGasto;
use Schaffen\Gef\Models\ProdutoFrota;
use Schaffen\Gef\Models\Veiculo;
use Schaffen\Gef\Repositories\ApontamentoVeiculoGastoRepository;
use Schaffen\Gef\Validators\ApontamentoVeiculoGastoValidator;

class ApontamentoVeiculoGastoService
{

   private $repository;
   private $validator;
   private $item_service;

   public function __construct(ApontamentoVeiculoGastoRepository $repository, ApontamentoVeiculoGastoValidator $validator, ApontamentoVeiculoGastoItemService $item_service, ApontamentoVeiculoGastoAnexoService $anexo_apontamento_service)
   {
      $this->repository = $repository;
      $this->validator = $validator;
      $this->item_service = $item_service;
      $this->anexo_apontamento_service = $anexo_apontamento_service;
   }

   public function updateOrCreate($data, $codigo = null)
   {
      try {
         \DB::beginTransaction();


         $attrib = ['avg_codigo' => $codigo];
         $validator_rules = is_numeric($codigo) ? ApontamentoVeiculoGastoValidator::RULE_UPDATE : ApontamentoVeiculoGastoValidator::RULE_CREATE;
         $this->validator->with($data)->passesOrFail($validator_rules);
         $this->repository->setPresenter("Schaffen\\Gef\\Presenters\\ApontamentoVeiculoGastoPresenter");

         $data_anexos = null;
         $data_item = $data['items'];
         unset($data['items']);

         if(isset($data['anexos'])){
            $data_anexos = $data['anexos'];
            unset($data['anexos']);
         }


         $veiculo_estoque = Veiculo::find($data['veiculo'])->local_estoque;
         $local_estoque_origem = isset($data['local_estoque']) ? $data['local_estoque'] : null;

         if ($local_estoque_origem) {
            $data['fk_tra_codigo'] = $this->transferirEstoque($veiculo_estoque, $local_estoque_origem, $data_item);

         } else {
            $this->lancaEstoque($veiculo_estoque, $data_item);
         }

         $data['fk_fornecedor_pes_codigo'] = isset($data['fornecedor']) ? $data['fornecedor'] : null;
         $data['fk_responsavel_pes_codigo'] = $data['responsavel'];
         $data['fk_vei_codigo'] = $data['veiculo'];

         $data = SCHArray::addPrefix(ApontamentoVeiculoGasto::$prefix, $data);

         $apontamento_gasto = $this->repository->updateOrCreate($attrib, $data);

         if (!is_numeric($codigo)) {

            foreach ($data_item as $item) {
               $item['apontamento_gasto'] = $apontamento_gasto['data']['codigo'];
               $this->item_service->updateOrCreate($item);
            }


            //insere anexos do apontamento
            if (isset($data_anexos) && count($data_anexos) > 0) {
               $anexos_service = new AnexoServices();
               $anexos_apontamento = $anexos_service->insertAnexos($data_anexos);

               if (!empty($anexos_apontamento)) {
                  foreach ($anexos_apontamento as $anexo) {
                     if ($anexo['new']) {
                        $anexo['apontamento_gasto'] = $apontamento_gasto['data']['codigo'];
                        $this->anexo_apontamento_service->updateOrCreate($anexo);
                     }
                  }
               }

            }
         }


         $response = [
            'status' => true,
            'errors' => '',
            'message' => 'Apontamento de gasto gravado.',
            'data' => $apontamento_gasto['data'],
            'http_code' => '200'
         ];
         \DB::commit();
         return $response;

      } catch (ValidatorException $e) {
         \DB::rollback();
         return $response = [
            'status' => false,
            'errors' => $e->getMessageBag(),
            'message' => 'Não foi possível gravar o Apontamento de gasto',
            'data' => null,
            'http_code' => '400'
         ];
      } catch (\Exception $e) {
         \DB::rollback();
         return $response = [
            'status' => false,
            'errors' => $e->getMessage() . $e->getTraceAsString(),
            'message' => 'Não foi possível gravar o Apontamento de gasto',
            'data' => null,
            'http_code' => '400'
         ];
      }
   }


   private function transferirEstoque($veiculo_estoque, $local_estoque_origem, $data_item)
   {

      $transferenciaDadosTabela = [
         'filial_origem' => null,
         'filial_destino' => $veiculo_estoque->loe_fk_fil_codigo,
         'codigo_origem' => $local_estoque_origem,
         'codigo_destino' => $veiculo_estoque->loe_codigo,
      ];
      $transferencia_service = new TransferenciaService();

      try {

         $transferencia = $transferencia_service->transferir($transferenciaDadosTabela);
         foreach ($data_item as $item) {
            $produto_frota = ProdutoFrota::find($item['produto']);
            $produto_variedade_embalagem = $produto_frota->produto_variedade_embalagem;
            $produto_produto = $produto_frota->produto;
            if ($produto_produto->pro_servico) {
               continue;
            }

            $transferencia_service->transferirItem($transferencia, [
               'valor_unitario' => $item['valor_unitario'],
               'pve_codigo' => $produto_variedade_embalagem->pve_codigo,
               'quantidade' => $item['quantidade'],

            ]);
         }

      } catch (\Exception $ex) {

         throw $ex;

      }

      return $transferencia->tra_codigo;

   }

   private function lancaEstoque($veiculo_estoque, $data_item)
   {

      $entrada_service = new EntradaService();


      $entradaDados = [
         "ent_fk_fil_codigo" => $veiculo_estoque->loe_fk_fil_codigo,
         "ent_fk_loe_codigo" => $veiculo_estoque->loe_codigo,
         "ent_fk_pes_codigo" => \Auth::user()->pessoa->pes_codigo,
      ];

      try {

         $entrada = $entrada_service->lancarEntrada($entradaDados);

         foreach ($data_item as $item) {


            $produto_frota = ProdutoFrota::find($item['produto']);
            $produto_variedade_embalagem = $produto_frota->produto_variedade_embalagem;
            $produto_variedade = $produto_variedade_embalagem->variedade;
            $produto_produto = $produto_frota->produto;
            if ($produto_produto->pro_servico) {
               continue;
            }

            $entrada_service->lancarEntradaItem($entrada, [
               "eni_valor_unitario" => $item['valor_unitario']
               , "eni_fk_pve_codigo" => $produto_variedade_embalagem->pve_codigo
               , "eni_quantidade" => $item['quantidade']
               , 'prv_codigo' => $produto_variedade->prv_codigo
            ]);
         }

      } catch (\Exception $ex) {

         throw $ex;

      }

      return $entrada->ent_codigo;

   }
}
